package
{
	import flash.display.*;
	import flash.events.*;
	import flash.media.*;
	
	public class AudioControl extends Sprite
	{
		[Embed(source="images/sound-on.png")]
		public static var soundOnImageSrc:Class;
		
		[Embed(source="images/sound-off.png")]
		public static var soundOffImageSrc:Class;
		
		[Embed(source="audio/desert.mp3")]
		public static var desertSrc:Class;
		
		[Embed(source="audio/tango.mp3")]
		public static var iceSrc:Class;
		
		[Embed(source="audio/boing.mp3")]
		public static var jumpSoundSrc:Class;
		
		[Embed(source="audio/scream.mp3")]
		public static var deathSoundSrc:Class;
		
		/*[Embed(source="audio/glass_break.mp3")]
		public static var glassBreakSoundSrc:Class;*/
		
		[Embed(source="audio/win.mp3")]
		public static var winSoundSrc:Class;
		
		private var soundOnImage : Bitmap;
		private var soundOffImage : Bitmap;
		
		private static var musicDesert : Sound;
		private static var musicIce : Sound;
		private static var music : Sound;
		private static var musicChannel : SoundChannel;
		
		private static var jumpSound : Sound;
		private static var jumpChannel : SoundChannel;
		
		private static var deathSound : Sound;
		private static var winSound : Sound;
		
		public static var mute : Boolean = false;
		
		public function AudioControl ()
		{
			soundOnImage = new soundOnImageSrc();
			soundOffImage = new soundOffImageSrc();
			
			addChild(soundOffImage);
			addChild(soundOnImage);
			
			soundOffImage.visible = mute;
			soundOnImage.visible = ! mute;
			
			addEventListener(MouseEvent.CLICK, toggleSound);
			
			buttonMode = true;
			mouseChildren = false;
			
			jumpSound = new jumpSoundSrc();
			deathSound = new deathSoundSrc();
			winSound = new winSoundSrc();
			
			musicDesert = new desertSrc();
			musicIce = new iceSrc();
			
			music = musicIce;
			
			musicChannel = music.play(0, int.MAX_VALUE);
		}

		public static function swapTheme (): void
		{
			if (musicChannel) {
				musicChannel.stop();
			}
			
			if (music == musicIce) {
				music = musicDesert;
			} else {
				music = musicIce;
			}
			
			if (! mute) {
				musicChannel = music.play(0, int.MAX_VALUE);
			}
		}

		public function toggleSound (e : Event) : void
		{
			mute = ! mute;
			
			soundOffImage.visible = mute;
			soundOnImage.visible = ! mute;
			
			if (mute) {
				if (musicChannel) {
					musicChannel.stop();
				}
			} else {
				musicChannel = music.play(0, int.MAX_VALUE);
			}
		}
		
		public static function startLevel () : void
		{
			if (mute) { return; }
			
			if (! musicChannel) {
				musicChannel = music.play(0, int.MAX_VALUE);
			}
		}
		
		public static function playJumpSound () : void
		{
			if (mute) { return; }
			
			if (jumpChannel) {
				jumpChannel.stop();
			}
			
			jumpChannel = jumpSound.play();
		}
		
		public static function playHitHeadSound () : void
		{
			// NOTHING
		}
		
		public static function playDeathSound () : void
		{
			if (mute) { return; }
			
			if (musicChannel) {
				musicChannel.stop();
				musicChannel = null;
			}
			
			deathSound.play();
		}
		
		public static function playWinSound () : void
		{
			if (mute) { return; }
			
			winSound.play();
		}
	}
}

