package
{
	import flash.display.*;
	import flash.geom.*;
	import flash.net.*;
	import flash.ui.*;
	
	import flash.filters.DisplacementMapFilter;
	import flash.filters.DisplacementMapFilterMode;
	
	public class Level extends Sprite
	{
		public var tiles : TileGrid;
		
		[Embed(source="levels/level1.txt", mimeType="application/octet-stream")]
		public static var level1file : Class;

		[Embed(source="levels/level2.txt", mimeType="application/octet-stream")]
		public static var level2file : Class;
		
		[Embed(source="levels/level3.txt", mimeType="application/octet-stream")]
		public static var level3file : Class;
		
		[Embed(source="levels/level4.txt", mimeType="application/octet-stream")]
		public static var level4file : Class;
		
		[Embed(source="levels/level5.txt", mimeType="application/octet-stream")]
		public static var level5file : Class;
		
		[Embed(source="levels/level6.txt", mimeType="application/octet-stream")]
		public static var level6file : Class;
		
		[Embed(source="levels/level7.txt", mimeType="application/octet-stream")]
		public static var level7file : Class;
		
		public static var levels : Array;
		
		public static const lastLevel : int = 7;
		
		[Embed(source="images/title.png")]
		public static var titleImage : Class;
		
		[Embed(source="images/congrats.png")]
		public static var congratsImage : Class;
		
		[Embed(source="images/desert.jpg")]
		public static var desertImg : Class;
		
		public var entities : Array;
		public var particles : Array;
		
		public var player : Player;
		
		public var levelID : int;
		
		private var entityOverlay : Sprite;
		private var particleOverlay : Sprite;
		
		private var _sourceBmp		:BitmapData;
		public var targetBmp	:BitmapData;
		private var perlinNoiseBmp	:BitmapData;
		private var displacementBmp	:BitmapData;
		
		public var bitmap:Bitmap;
			
		private var _offsetArray:Array = [new Point(), new Point()];
		private var _displacementFilter:DisplacementMapFilter;
		
		public var mirror: Point;
		public var mirrorNormal: Point;
		
		public var editing: Boolean = false;
		public var editTile: int = 1;
		public var editImage: Bitmap = new Bitmap();
		
		public static function initLevels () : void
		{
			levels = new Array();
			
			levels[1] = new level1file();
			levels[2] = new level2file();
			levels[3] = new level3file();
			levels[4] = new level4file();
			//levels[5] = new level5file();
			levels[5] = new level6file();
			levels[6] = new level7file();
		}
		
		public static function create (i : int = 1) : Level
		{
			return new Level(levels[i].toString(), i);
		}
		
		public function Level (data: String, i: int = 1)
		{
			editImage.alpha = 0;
			
			levelID = i;
			
			if (i == 6) {
				mirror = new Point(320, 240);
				mirrorNormal = new Point(0, -1);
			} else if (i == 4) {
				mirror = new Point(120, 200);
				mirrorNormal = new Point(-Math.cos(124), -Math.sin(124));
			} else {
				mirror = new Point(320, 240);
				mirrorNormal = new Point(1, 0);
			}
			
			initFilter();
			
			if (levelID == 1) {
				addChild(new titleImage());
			} else if (levelID == lastLevel) {
				addChild(new congratsImage());
			}
			
			tiles = new TileGrid(data);
			addChild(tiles);
			
			entityOverlay = new Sprite();
			addChild(entityOverlay);
			entities = new Array();
			
			particleOverlay = new Sprite();
			addChild(particleOverlay);
			particles = new Array();
			
			Player.reachedExit = false;
			addChild(editImage);
		}
		
		
		private function initFilter (): void {
			if (Tile.tileBitmapData == Tile.tileBitmapDataIce) {
				_sourceBmp = (new Images.background()).bitmapData;
			} else {
				_sourceBmp = (new desertImg()).bitmapData;
			}
			
			var overlay: Shape = new Shape();
			
			overlay.graphics.beginFill(0x00FF80, 0.25);
			overlay.graphics.moveTo(mirror.x, mirror.y);
			overlay.graphics.lineTo(mirror.x + mirrorNormal.y * 1000, mirror.y - mirrorNormal.x * 1000);
			overlay.graphics.lineTo(mirror.x + mirrorNormal.x * 1000, mirror.y + mirrorNormal.y * 1000);
			overlay.graphics.lineTo(mirror.x - mirrorNormal.y * 1000, mirror.y + mirrorNormal.x * 1000);
			overlay.graphics.endFill();
			
			_sourceBmp.draw(overlay, new Matrix(1, 0, 0, 1, 0, 0));
			
			bitmap = new Bitmap(_sourceBmp);
			
			addChild(bitmap);
			
			return;
			
			var bounds: Rectangle = new Rectangle(0, 0, 640, 480);
			
			var offset_y:Number = 0;
			var offset_x:Number = 0;
			
			var w:Number = bounds.width + offset_x;
			var h:Number = bounds.height + offset_y;
			
			
			targetBmp = new BitmapData(w, h, true, 0);
			displacementBmp = new BitmapData(w, h);
			perlinNoiseBmp = new BitmapData(64, 64);
			
			displacementBmp.fillRect(new Rectangle(0, 0, 640, 480), 0x80808080);
			
			bitmap = new Bitmap(targetBmp);
			//_sourceBmp.draw(new Images.background(), new Matrix(1, 0, 0, 1, bounds.x * -1, bounds.y * -1));
			//_sourceBmp.draw(this, new Matrix(0.25, 0, 0, 0.25, bounds.x * -1, bounds.y * -1));
			
			/*bitmap.scaleX = 4;
			bitmap.scaleY = 4;*/
			
			this.addChild(bitmap)
			/*bitmap.x = logo_mc.x + bounds.x;
			bitmap.y = logo_mc.y + bounds.y;*/
			
			_displacementFilter = new DisplacementMapFilter(displacementBmp, new Point(), 1, 1, 16, 16,DisplacementMapFilterMode.COLOR);
		
			//this.addChild(new Bitmap(displacementBmp))
		
			//logo_mc.filters = [glow2];
			//bitmap.blendMode = BlendMode.SCREEN;
			//bitmap.filters = [glow2, glow3];
		}
		
		public function update (dt: int): void
		{
			if (editing) {
				var ix: int = mouseX / Tile.WIDTH;
				var iy: int = mouseY / Tile.HEIGHT;
				editImage.alpha = 1;
				
				editImage.bitmapData = Tile.tileBitmapData[editTile];
				editImage.x = ix * Tile.WIDTH;
				editImage.y = iy * Tile.HEIGHT;
				
				if (Input.isMouseDown) {
					tiles.put(Tile.create(editTile), ix, iy);
				}
				
				return;
			} else {
				editImage.alpha = 0;
			}
			
			Player.touchingTargets = 0;
			
			tiles.update(dt);
			
			for (var i : int = 0; i < entities.length; i++)
			{
				entities[i].update(dt);
			}
			
			for (i = 0; i < particles.length; i++)
			{
				particles[i].update(dt);
			}
			
			particles = particles.filter(removeParticlesFilter);
		}
		
		public function render (): void {
			/*if (! player) { return; }
			
			_offsetArray[0].x -= 2.5;
			_offsetArray[1].x -= 3.5;
			perlinNoiseBmp.perlinNoise(30, 30, 3, 164, true, true, 1, true, _offsetArray);
			
			var p: Point = new Point(320, player.y - 32);
			
			displacementBmp.copyPixels(perlinNoiseBmp, perlinNoiseBmp.rect, p);
			
			displacementBmp.fillRect(new Rectangle(0, 0, 320, 480), 0x80808080);
			
			targetBmp.applyFilter(_sourceBmp, _sourceBmp.rect, new Point(), _displacementFilter);
			
			displacementBmp.fillRect(new Rectangle(p.x, p.y, 64, 64), 0x80808080);*/
		}
		
		public function addEntity (entity : DisplayObject) : void
		{
			entities.push(entity);
			
			if (entity is Player)
			{
				player = entity as Player;
				entityOverlay.addChild(player.reflection);
			}
			
			entityOverlay.addChild(entity);
		}
		
		private function removeParticlesFilter (element : *, index: int, arr: Array): Boolean
		{
			if (element.dead())
			{
				particleOverlay.removeChild(element);
				
				return false;
			}
			else
			{
				// Element remains in array
				return true;
			}
		}
		
		public function spark (x : Number, y : Number, num : int, lifetime : int = 350) : void
		{
			var s : Spark;
			
			for (var i : int = 0; i < num; i++)
			{
				s = new Spark(x, y, lifetime);
				
				particleOverlay.addChild(s);
				
				particles.push(s);
			}
		}
		
		private static function returnTrue (foo : *) : Boolean
		{
			return true;
		}
		
		public function get (entity: *, testFunc : Function = null) : Array
		{
			if (testFunc == null) { testFunc = returnTrue; }
			
			var rect : Rectangle = entity.getAABB();
			
			var array : Array = new Array();
			
			var ix1 : int = rect.left / Tile.WIDTH;
			var ix2 : int = rect.right / Tile.WIDTH;
			
			var iy1 : int = rect.top / Tile.HEIGHT;
			var iy2 : int = rect.bottom / Tile.HEIGHT;
			
			for (var ix : int = ix1; ix <= ix2; ix++)
			{
				for (var iy : int = iy1; iy <= iy2; iy++)
				{
					var tile : * = tiles.getFromIndex(ix, iy);
					
					if (tile && tile != entity 
						&& tile.getAABB().intersects(rect)
						&& testFunc(tile))
					{
						array.push(tile);
					}
				}
			}
			
			for (var i : int = 0; i < entities.length; i++)
			{
				if (entities[i] != entity
					&& entities[i].getAABB().intersects(rect)
					&& testFunc(entities[i])
					)
				{
					array.push(entities[i]);
				}
			}
			
			return array;
		}
	}
}

