package
{
	import flash.display.*;
	import flash.events.*;
	import flash.net.*;
	import flash.ui.*;
	import flash.utils.getTimer;
	
	[SWF (width=640, height=480, frameRate=60)]
	public class Main extends Sprite
	{
		private var time: int;

		public static var level: Level;
		
		private var oldLevel: Level;
		
		public static var audioControl : AudioControl;
		
		private var delay : int = 0;
		
		public function Main ()
		{
			addChild(new Images.background());
			
			Level.initLevels();
			
			Tile.initTiles();
			
			level = Level.create();
			
			addChild(level);
			
			audioControl = new AudioControl();
			
			addChild(audioControl);
			
			if (stage)
			{
				init();
			}
			else
			{
				addEventListener(Event.ADDED_TO_STAGE, init);
			}
		}
		
		public function init (param : * = 0) : void
		{
			Input.attach(stage);
			
			stage.addEventListener( KeyboardEvent.KEY_DOWN, keyDownListener, false, 0, true );
			
			time = getTimer();
			
			stage.addEventListener(Event.ENTER_FRAME, onEnterFrame);
		}
		
		private function onEnterFrame(e:Event):void
		{
			var dt:int = getTimer() - time;
			
			while (dt >= 20) {
				time += 20;
				dt -= 20;
				
				update(20);
			}
			level.render();
		}
		
		private function update (dt: int): void
		{
			if (Player.dead || Player.reachedExit) {
				delay += dt;
			}
			
			if (Player.dead && delay > 1500) {
				resetLevel();
				delay = 0;
			}
			
			if (Player.reachedExit && !oldLevel && delay > 1500) {
				nextLevel();
				delay = 0;
			}
			
			if (oldLevel)
			{
				oldLevel.y += 10;
				level.y += 10;
				
				if (level.y >= 0)
				{
					level.y = 0;
					removeChild(oldLevel);
					oldLevel = null;
				}
			} else {
				level.update(dt);
			}
		}
		
		public function resetLevel () : void
		{
			var levelID : int = level.levelID;
			
			AudioControl.startLevel();
			
			removeChild(level);
			level = Level.create(levelID);
			addChild(level);
			
			swapChildren(level, audioControl);
		}
		
		public function nextLevel () : void
		{
			var levelID : int = level.levelID;
			
			if (levelID == Level.lastLevel)
			{
				levelID = 1;
				/*Tile.swapTheme();
				AudioControl.swapTheme();*/
			}
			else
			{
				levelID++;
				
				if (levelID == Level.lastLevel || levelID == 4) {	
					Tile.swapTheme();
					AudioControl.swapTheme();
				}
			}
			
			setNextLevel(Level.create(levelID));
		}
		
		public function setNextLevel (_level: Level) : void
		{
			AudioControl.startLevel();
			
			oldLevel = level;
			
			level = _level;
			level.y = -480;
			
			addChild(level);
			
			swapChildren(level, audioControl);
		}
		
		private var file: FileReference;
		
		private function keyDownListener( ev:KeyboardEvent ):void
		{
			var modifier: Boolean = ev.ctrlKey || ev.shiftKey;
			
			if (ev.keyCode == 69) {
				level.editing = ! level.editing;
			}
			
			if (ev.keyCode == 33) { // page up
				level.editTile = (level.editTile + 1) % Tile.MAX;
			}
			
			if (ev.keyCode == 34) { // page down
				level.editTile = (level.editTile == 0) ? Tile.MAX : level.editTile - 1;
			}
			
			if (ev.keyCode == 79 && modifier) {
				file = new FileReference();
				file.addEventListener(Event.SELECT, fileSelect);
				file.browse();
			}
			
			if (ev.keyCode == 83 && modifier) {
				new FileReference().save(level.tiles.toString());
			}
		}

		private function fileSelect (event:Event):void {
			file.addEventListener(Event.COMPLETE, loadComplete);
			file.load();
		}

		private function loadComplete (event:Event):void {
			setNextLevel(new Level(file.data.toString(), 1));
		}

	}
}
