package
{
	import flash.display.*;
	import flash.geom.*;
	import flash.ui.*;
	import gs.*;
	import gs.easing.*;
	
	public class Player extends Sprite
	{
		[Embed(source="images/character.png")]
		public static var characterImg : Class;
		
		[Embed(source="images/reflection.png")]
		public static var reflectionImg : Class;
		
		public static var touchingTargets: int = 0;
		public static var playerCount: int = 1;
		
		public var facing : int;
		public var offset: Number = 16;
		
		public var tileID: int;
		
		private var image : Animation;
		private var reflectImage : Animation;
		
		private var vy : Number = 0;
		
		private var standing : Boolean = false;
		private var jumpCancellable : Boolean = false;
		
		public static var dead : Boolean = false;
		public static var reachedExit : Boolean = false;
		public var dead2 : Boolean = false;
		
		private var keyLeft: uint;
		private var keyRight: uint;
		private var keyJump: uint;
		
		public var reflection: Sprite = new Sprite();
		
		public function Player (_x : Number, _y : Number, type: int)
		{
			tileID = type;
			
			keyLeft = Keyboard.LEFT;
			keyRight = Keyboard.RIGHT;
			facing = 1;
			
			keyJump = 90;//Keyboard.UP;
			
			image = new Animation(new characterImg(), 150);
			
			addChild(image);
			
			reflectImage = new Animation(new reflectionImg(), 150);
			
			reflection.addChild(reflectImage);
			
			x = _x + 8;
			y = _y - 16;
			
			dead = false;
			
			image.x = -offset;
			reflectImage.x = -offset;
		}
		
		public function getAABB () : Rectangle
		{
			return new Rectangle(x - 8, y, 16, 32);
		}
		
		private function isSolid (obj : *) : Boolean
		{
			return (obj is SpawnTile || obj.tileID == Tile.SPIKES || obj.tileID == Tile.SPIKES2 || obj.tileID == Tile.EXIT) ? false : true;
		}
		
		private function isDeadly (obj : *) : Boolean
		{
			return (obj is StaticTile && (obj.tileID == Tile.SPIKES || obj.tileID == Tile.SPIKES2));
		}
		
		private function isTarget (obj : *) : Boolean
		{
			return (obj is Balloons);
		}
		
		public function checkX () : void
		{
			var playerRect : Rectangle = getAABB();
			
			var entities : Array = Main.level.get(this, isSolid);
			
			var overlapLeft : Number = 0;
			var overlapRight : Number = 0;
			
			for (var i : int = 0; i < entities.length; i++)
			{
				var otherRect : Rectangle = entities[i].getAABB();
				
				if (playerRect.bottom <= otherRect.top || otherRect.bottom <= playerRect.top) { continue; }
				
				var a : Number = playerRect.right - otherRect.left;
				var b : Number = otherRect.right - playerRect.left;
				
				if (a < b)
				{
					overlapRight = Math.max(a, overlapRight);
				} else {
					overlapLeft = Math.max(b, overlapLeft);
				}
			}
			
			if (overlapLeft > overlapRight) {
				x += overlapLeft;
			} else {
				x -= overlapRight;
			}
		}
		
		public function checkY () : void
		{
			var playerRect : Rectangle = getAABB();
			
			var entities : Array = Main.level.get(this, isSolid);
			
			var overlapTop : Number = 0;
			var overlapBottom : Number = 0;
			
			for (var i : int = 0; i < entities.length; i++)
			{
				var otherRect : Rectangle = entities[i].getAABB();
				
				if (playerRect.left >= otherRect.right || otherRect.left >= playerRect.right) { continue; }
				
				var a : Number = playerRect.bottom - otherRect.top;
				var b : Number = otherRect.bottom - playerRect.top;
				
				if (a < b)
				{
					overlapBottom = Math.max(a, overlapBottom);
				} else {
					overlapTop = Math.max(b, overlapTop);
				}
			}
			
			if (overlapTop > overlapBottom) { // hit his head
				y += overlapTop;
				vy = Math.max(vy, 0);
				Main.level.spark(x, y + 4, 1);
				AudioControl.playHitHeadSound();
			} else if (overlapBottom > 0) { // standing on floor
				y -= overlapBottom;
				vy = Math.min(vy, 0);
				standing = true;
			}
		}
		
		public function update (dt: int): void
		{
			if (! dead) {
				image.update(20);
			}
			
			if (! dead) {
				reflectImage.update(20);
			}
			
			if (! dead && ! reachedExit)
			{
				var dir : int = 0;
				if (Input.keyPressed(keyRight)) { dir += 1; }
				if (Input.keyPressed(keyLeft)) { dir -= 1; }
				
				if (dir != 0) {
					facing = dir;
				}
				
				scaleX = facing;
				//image.x = facing * -offset;
				
				x += dir * 0.15 * dt;
				
				checkX();
			}
			
			var a : Number = 0;
			
			var jumpPressed: Boolean = (Input.keyPressed(90) || Input.keyPressed(Keyboard.UP) || Input.keyPressed(Keyboard.SPACE));
			
			if (jumpPressed && standing && !dead && ! reachedExit)
			{
				vy = -0.4;
				standing = false;
				AudioControl.playJumpSound();
				jumpCancellable = true;
			} else if (jumpCancellable && ! jumpPressed && ! standing && !dead && vy < 0) {
				vy *= 0.5;
				jumpCancellable = false;
			}
			
			a = 0.001;
			vy += a * dt;
			
			y += vy * dt - 0.5 * a * dt * dt;
			
			standing = false;
			
			checkY();
			
			if (!dead && Main.level.get(this, isDeadly).length > 0)
			{
				dead = true;
				dead2 = true;
				AudioControl.playDeathSound();
			}
			
			if (dead2)
			{
				if (Math.random() < 0.5)
				{
					Main.level.spark(x, y + 12, Math.random() * 3, 1000)
				}
			}
			
			if (! reachedExit && Main.level.get(this, isTarget).length > 0) {
				reachedExit = true;
				AudioControl.playWinSound();
				
				/*TweenLite.to(this, 1.0, {ease: Linear.easeNone, alpha: 1});
				TweenLite.to(reflection, 1.0, {ease: Linear.easeNone, x: this.x, scaleX: this.scaleX});*/
			}
			
			const MARGIN: Number = 64;
			
			// centre of mirror
			var mcx: Number = Main.level.mirror.x;
			var mcy: Number = Main.level.mirror.y;
			
			// normal of mirror
			var mnx: Number = Main.level.mirrorNormal.x;
			var mny: Number = Main.level.mirrorNormal.y;
			
			//Main.level.bitmap.alpha = 0;
			
			var mirrorDist: Number = (mcx - x) * mnx + (mcy - y) * mny;
			
			if (dead || reachedExit) {
				alpha += 0.02;
				//reflection.alpha = 1 - alpha;
			} else if (mirrorDist < -MARGIN) {
				alpha = 0.02;
				reflection.alpha = 1;
			} else if (MARGIN < mirrorDist) {
				alpha = 1;
				reflection.alpha = 0;
			} else {
				alpha = 0.5 + mirrorDist / (MARGIN * 2);
				reflection.alpha = 1 - alpha;
				
				//Main.level.bitmap.alpha = 1 - Math.abs(mirrorDist) / MARGIN;
				
				/*if (0 < mirrorDist) {
					alpha = 1;
					reflection.alpha = 1 - mirrorDist / MARGIN;
				} else {
					alpha = 1 + mirrorDist / MARGIN;
					reflection.alpha = 1;
				}*/
			}
			
			reflection.x = x + mirrorDist * (mnx) * 2;
			reflection.y = y + mirrorDist * (mny) * 2;
			reflection.rotation = Math.atan2(mny, mnx) * 180 / 3.1415 * 2;
			
			if (! (dead || reachedExit)) {
				reflection.scaleX = -facing;
				//reflectImage.x = facing * offset;
			}
		}
	}
	
}

